<#compress>

<#-- Import common macros and functions -->
<#import "../../application/docbook+xml/common_module_traversal_utils.ftl" as utils>
<#import "../../application/docbook+xml/macros_common_general.ftl" as com>

<#-- Initialize common variables -->
<@com.initializeMainVariables/>

<#if _subject.submissionType?has_content && _subject.submissionType?matches("EU_PPP_.*")>
    <@com.initiRelevanceForPPP relevance/>
</#if>

<#-- Root entity -->    
<#assign rootEntity = entity.root>

<#-- Define the main macro and the arguments to be used to get output using the traversal approach -->
<#assign mainMacroName = "findStudies"/>
<#assign mainMacroArgs = ['docElementNode', 'sectionDoc', 'sectionNode', 'entityDocument', 'level']/>

<#-- Make global the macroCall as well as each macro used in it -->
<#global mainMacroCall="<@" + mainMacroName + " " + mainMacroArgs?join(" ") + "/>"/>
<#global findStudies = findStudies/>

<#-- Print CSV header row -->
<@printCsvHeadersRow />

<#-- variables to be used in the code -->
<#assign selectedDocTypes = ["ENDPOINT_STUDY_RECORD"]/>

<#-- make link to specifications on IUCLID webpage -->
<#assign resultsPathList = ["ResultsAndDiscussion.Partcoeff", "ResultsAndDiscussion.WaterSolubility", "ResultsAndDiscussion.EffectConcentrations", "ResultsAndDiscussion.EffectLevels", "ResultsAndDiscussion.EffectLevels.Efflevel", "ResultsAndDiscussion.Absorption", "ResultsAndDiscussion.ResultsFetuses.EffectLevelsFetuses.Efflevel", "ResultsAndDiscussion.ResultsMaternalAnimals.EffectLevelsMaternalAnimals.Efflevel", "ResultsAndDiscussion.InVitro.ResultsOfExVivoInVitroStudy", "ResultsAndDiscussion.InVivo.IrritationCorrosionResults", "ResultsAndDiscussion.TestRs", "ResultsAndDiscussion.InVitro.Results", "25", "ResultsAndDiscussion.InVitroInChemico.Results", "ResultsAndDiscussion.TraditionalSensitisationTest.ResultsOfTest", "ResultsAndDiscussion.ResultsF2Generation.EffectLevelsF2.Efflevel", "ResultsAndDiscussion.ResultsOfExaminationsOffspring.EffectLevelsF1.Efflevel", "ResultsAndDiscussion.ResultsOfExaminationsParentalGeneration.EffectLevelsP0.Efflevel", "ResultsAndDiscussion.ResultsP1SecondParentalGeneration.EffectLevelsP1.Efflevel"]/>

<#-- Do the traverse -->
<@utils.traversal rootEntity/>

<#----------------------------------------------->
<#-- Macro and function definitions start here -->
<#----------------------------------------------->
<#-- define header section -->
<#macro printCsvHeadersRow>"Entity Type","Entity Name","Section Name","Section Number","Document Name","Document UUID","Waiver","Reference Substance Name","Reference Substance CAS Nr","Reference Substance EC Nr","Reference Substance IUPAC and Ichi","Robust Study Summary","Adequacy of Study","Study period","Type of information","Reliability","Data access","Guideline (materials and methods)","GLP compliant","Test Material Information","Test organisms","Strain / cell type","Route of application / dose method","Exposure duration","Metabolic activation","Metabolic activation system","Key Results","Additional Results","Literature Reference Type","Literature Reference Title","Literature Reference Author","Literature Reference Year","Literature Reference Source","Literature Reference Testing Facility","Literature Reference Report Date","Literature Reference Report Number","Literature Reference Study Sponsor","Literature Reference Study Number","Literature Reference Remarks","Literature Reference Other Study Identifiers","Literature Reference Attachments","Dossier Name","Dossier Creation Date","IUCLID Version"
</#macro>

<#-- call findStudies and determine the elements to search through -->
<#macro findStudies docElementNode sectionDoc sectionNode entityDocument level>

    <#-- In order to be able to skip unnecessary traversals, we only look at document level (level==0) and check if the document type is in the list
    of selected document types and it has any content (so we know it makes sense to go through the node) -->
    <#if level==0 && selectedDocTypes?seq_contains(sectionDoc.documentType) && docElementNode?has_content>

        <#-- retrieve all literature references for the current node (function defined below) -->
        <#local litRefs = getLiteratureReferences(sectionDoc)/>

        <#-- if there are any literature references -->
        <#if litRefs?has_content>
            <#list litRefs as litRef> 
                <@tableOutput litRef entityDocument sectionNode sectionDoc docElementNode  />
            </#list>
        <#else>
            <#local litRef=""/>
            <@tableOutput litRef entityDocument sectionNode sectionDoc docElementNode  />
        </#if>

    </#if>
</#macro>

<#-- a function to properly create entries as needed: one, if there is one reference, multiple, if there are more than one -->
<#function getLiteratureReferences docElementNode>

    <#local listOfLiteratureReferences = []/>

    <#-- if it is a node with a single entry -->
    <#if docElementNode?node_type == "document_reference">
        <#local listOfLiteratureReferences = checkAndAddLitReference(docElementNode, listOfLiteratureReferences)/>
    </#if>

    <#-- if it is a node with multiple entries (denote the "s" in the document_reference*s* !) -->
    <#if docElementNode?node_type == "document_references">
        <#list docElementNode as ref>
            <#local listOfLiteratureReferences = checkAndAddLitReference(ref, listOfLiteratureReferences)/>
        </#list>
    </#if>

    <#if docElementNode?children?has_content>
        <#list docElementNode?children as child>
            <#if child?has_content>
                <#local listOfLiteratureReferences = listOfLiteratureReferences + getLiteratureReferences(child)/>
            </#if>
        </#list>
    </#if>

    <#return listOfLiteratureReferences>

</#function>

<#function checkAndAddLitReference key listOfLiteratureReferences>
    <#local doc = iuclid.getDocumentForKey(key) />
    
    <#if doc?has_content && doc.documentType == "LITERATURE">
        <#return listOfLiteratureReferences + [doc]>
    <#else>
        <#return listOfLiteratureReferences/>
    </#if>
    
</#function>

<#macro tableOutput literatureRef entityDocument sectionNode sectionDoc docElementNode>
    
    <#-- Data waiving -->
    <#local waiver><#compress>
        <#if sectionDoc.hasElement("AdministrativeData.DataWaiving")>
            <#-- retrieve the value from the waiver picklist (.value from common modules takes care of data types) -->
            <@com.value sectionDoc.AdministrativeData.DataWaiving/> 
        <#else>
            field not available
        </#if>
    </#compress></#local>

    <#-- robust study summary -->
    <#local robustStudySummaryBoolean><#compress>
        <#if sectionDoc.hasElement("AdministrativeData.RobustStudy")>
            <#if sectionDoc.AdministrativeData.RobustStudy>
                Yes
            <#else>
                No
            </#if>
        <#else>
            field not available
        </#if>
    </#compress></#local>

    <#-- adequacy -->
    <#local adequacyOfStudy><#compress>
        <#if sectionDoc.hasElement("AdministrativeData.PurposeFlag")>
            <@com.value sectionDoc.AdministrativeData.PurposeFlag />
        <#else>
            field not available
        </#if>
    </#compress></#local>

    <#-- adequacy -->
    <#local studyPeriod><#compress>
        <#if sectionDoc.hasElement("AdministrativeData.StudyPeriod")>
            <#if sectionDoc.hasElement("AdministrativeData.StudyPeriodStartDate")>
                <#if sectionDoc.AdministrativeData.StudyPeriodStartDate?has_content && sectionDoc.AdministrativeData.StudyPeriodEndDate?has_content>
                    <@com.value sectionDoc.AdministrativeData.StudyPeriodStartDate/> - <@com.value sectionDoc.AdministrativeData.StudyPeriodEndDate/>
                </#if>
            </#if>
            <@com.value sectionDoc.AdministrativeData.StudyPeriod />
        <#else>
            field not available
        </#if>
    </#compress></#local>

    <#-- type of information -->
    <#local typeOfInformation><#compress>
        <#if sectionDoc.hasElement("AdministrativeData.StudyResultType")>
            <@com.value sectionDoc.AdministrativeData.StudyResultType />
        <#else>
            field not available
        </#if>
    </#compress></#local>

    <#-- reliability -->
    <#local reliabilityScore><#compress>
        <#if sectionDoc.hasElement("AdministrativeData.Reliability")>
            <@com.value sectionDoc.AdministrativeData.Reliability />
        <#else>
            field not available
        </#if>
    </#compress></#local>

    <#-- data access -->
    <#local dataAccess><#compress>
        <#if sectionDoc.hasElement("DataSource.DataAccess")>
            <@com.value sectionDoc.DataSource.DataAccess />
        <#else>
            field not available
        </#if>
    </#compress></#local>

    <#-- guideline -->
    <#local guidelineUsed = getGuidelines(sectionDoc)/>
    
    <#-- GLP -->
    <#local glpCompliant><#compress>
        <#if sectionDoc.hasElement("MaterialsAndMethods.GLPComplianceStatement")>            
            <@com.value sectionDoc.MaterialsAndMethods.GLPComplianceStatement />
        <#else>
            field not available
        </#if>
    </#compress></#local>

    <#-- Test material name and cas number of linked reference substance(s) -->
    <#local testMaterialInformation = getTestMaterialInformation(sectionDoc)/>
    
    <#-- Dossier details -->
    <#local dossierName><@utils.csvValue _dossierHeader.name/></#local>
    <#local dossierCreationDate><@utils.csvValue _dossierHeader.creationDate/></#local>
    
    <#-- Url -->
    <#local docUrl=utils.getDocumentUrl(sectionDoc)/>

    <#-- species *note that only species in materials and methods section are extracted, e.g. not strain/species in genetic tox in vitro which is in the results -->
    <#local species = getSpecies(sectionDoc)/>
       
    <#-- strainOrCell -->
    <#local strainOrCell><#compress>
        <#if sectionDoc.hasElement("MaterialsAndMethods.TestAnimals.Strain")>
            <@com.value valuePath=sectionDoc.MaterialsAndMethods.TestAnimals.Strain printDescription=false/>  
        <#else>
            field not available
        </#if>
    </#compress></#local>
    
    <#-- route of application / dose method -->
    <#local routeApplicationDoseMethod = getRouteApplicationDoseMethod(sectionDoc)/>

    <#-- exposure duration -->
    <#local exposureDuration = getExposureDuration(sectionDoc)/>

    <#-- metabolic activation and metabolic activation system -->
    <#local metAct><#compress>
        <#if sectionDoc.hasElement("MaterialsAndMethods.Method.MetabolicActivation")>
            <@com.value sectionDoc.MaterialsAndMethods.Method.MetabolicActivation/>
        <#else>
            field not available
        </#if>
    </#compress></#local>
    
    <#local metActSystem><#compress>
        <#if sectionDoc.hasElement("MaterialsAndMethods.Method.MetabolicActivationSystem")>
            <@com.value sectionDoc.MaterialsAndMethods.Method.MetabolicActivationSystem/>
        <#else>
            field not available
        </#if>
    </#compress></#local>
        
    <#-- Study results: Effect levels -->
    <#local studyResults = getStudyResults(sectionDoc, resultsPathList)/>

    <#--  get all literature references fields in a hash  -->
    <#local literatureReferenceFields = getLitRefFields(literatureRef)/>

    <#-- reference substance ids -->
    <#if entityDocument.documentType=="SUBSTANCE">       

          <#if entityDocument?has_content>
            <#local refSubName><@com.referenceSubstanceName com.getReferenceSubstanceKey(entityDocument.ReferenceSubstance.ReferenceSubstance) />	</#local>

            <#local casNr><@com.casNumber com.getReferenceSubstanceKey(entityDocument.ReferenceSubstance.ReferenceSubstance) /></#local>
            <#local ecNr><@com.inventoryECNumber com.getReferenceSubstanceKey(entityDocument.ReferenceSubstance.ReferenceSubstance) /></#local>
            
            <#local iupacName><@com.iupacName com.getReferenceSubstanceKey(entityDocument.ReferenceSubstance.ReferenceSubstance) /></#local>
            <#local inchi><@com.inchi com.getReferenceSubstanceKey(entityDocument.ReferenceSubstance.ReferenceSubstance) /></#local>

            <#local iupacAndInchi = [iupacName, inchi]/>
            <#local iupacAndInchi = iupacAndInchi?join("; ")/>
        <#else>
            <#local refSubName=""/>
            <#local casNr=""/>
            <#local ecNr=""/>
            <#local iupacAndInchi = ""/>
        </#if>
    <#else>
        <#local refSubName="field not available"/>
        <#local casNr="field not available"/>
        <#local ecNr="field not available"/>
        <#local iupacAndInchi = "field not available"/>
    </#if> 

    <#-- print final input row -->
    <#compress>
    <@utils.csvValue entityDocument.documentType/>,<@utils.csvValue entityDocument.name/>,<@utils.csvValue sectionNode.title/>,${sectionNode.number},<@utils.csvValue sectionDoc.name/>,"=HYPERLINK(""${docUrl}"",""${sectionDoc.documentKey.uuid}"")",${waiver},<@utils.csvValue refSubName/>,<@utils.csvValue casNr/>,<@utils.csvValue ecNr/>,<@utils.csvValue iupacAndInchi/>,<@utils.csvValue robustStudySummaryBoolean/>,<@utils.csvValue adequacyOfStudy/>,<@utils.csvValue studyPeriod />,<@utils.csvValue typeOfInformation/>,<@utils.csvValue reliabilityScore />,<@utils.csvValue dataAccess/>,<@utils.csvValue guidelineUsed />,<@utils.csvValue glpCompliant />,<@utils.csvValue testMaterialInformation/>,<@utils.csvValue species />,<@utils.csvValue strainOrCell/>,<@utils.csvValue routeApplicationDoseMethod/>,<@utils.csvValue exposureDuration/>,<@utils.csvValue metAct/>,<@utils.csvValue metActSystem/>,<@utils.csvValue studyResults['key results']/>,<@utils.csvValue studyResults['additional results']/>,<@utils.csvValue literatureReferenceFields['LiteratureType']/>,<@utils.csvValue literatureReferenceFields['Name']/>,<@utils.csvValue literatureReferenceFields['Author']/>,<@utils.csvValue literatureReferenceFields['ReferenceYear']/>,<@utils.csvValue literatureReferenceFields['Source']/>,<@utils.csvValue literatureReferenceFields['TestLab']/>,<@utils.csvValue literatureReferenceFields['ReportDate']/>,<@utils.csvValue literatureReferenceFields['ReportNo']/>,<@utils.csvValue literatureReferenceFields['CompanyOwner']/>,<@utils.csvValue literatureReferenceFields['CompanyOwnerStudyNo']/>,<@utils.csvValue literatureReferenceFields['Remarks']/>,<@utils.csvValue literatureReferenceFields['StudyIdentifiers']/>,<@utils.csvValue literatureReferenceFields['Attachments']/>,${dossierName},${dossierCreationDate},${iuclid6Version!}
    </#compress>


</#macro>

</#compress>

<#-------------------------------------------------------------->
<#-- FUNCTIONS to retrieve fields from study record ------------>
<#-------------------------------------------------------------->

<#function getGuidelines doc>

    <#local guidelineList = []/>
         
    <#if doc.hasElement("MaterialsAndMethods.Guideline")>
        <#list doc.MaterialsAndMethods.Guideline as guidelineRow>
            <#if guidelineRow.Guideline?has_content>
                <#local guideline><@com.value guidelineRow.Guideline/></#local>
                <#local guidelineList = guidelineList + [guideline]/>
            </#if>
        </#list>
    </#if>
    
    <#return guidelineList?join(" | ")/>
</#function>


<#function getTestMaterialInformation doc>
    
    <#if doc.hasElement("MaterialsAndMethods.TestMaterials.TestMaterialInformation")>
        
        <#local testMaterialEntity = iuclid.getDocumentForKey(doc.MaterialsAndMethods.TestMaterials.TestMaterialInformation) />
        
        <#if testMaterialEntity?has_content> 

            <#-- local variable to capture test material name -->
            <#local testMaterialName><@com.value testMaterialEntity.Name /></#local>
            
            <#-- get test material reference substance CAS nbs -->
            <#local testMaterialReferenceSubstances = []/>
            <#if testMaterialEntity.Composition.CompositionList?has_content>
                <#list testMaterialEntity.Composition.CompositionList as component>
                    <#if component?has_content && component.ReferenceSubstance?has_content>
                
                        <#-- capture the cas number of the linked reference substance of the test material, and if it exists, append to list -->
                        <#local testMaterialReferenceSubstance><@com.casNumber com.getReferenceSubstanceKey(component.ReferenceSubstance) /></#local>
                        <#if testMaterialReferenceSubstance?has_content>
                            <#local testMaterialReferenceSubstances = testMaterialReferenceSubstances + [testMaterialReferenceSubstance]/>
                        </#if>
                    </#if>
                </#list>        
            </#if>

            <#--  create test material information as Test Material NAme (cas nbs)  -->
            <#if testMaterialReferenceSubstances?has_content>
                <#local testMaterialInformation>${testMaterialName} (${testMaterialReferenceSubstances?join("; ")})</#local>
            <#else>
                <#local testMaterialInformation = testMaterialName/>
            </#if>            
            
        <#else>
            <#local testMaterialInformation=""/>

        </#if>
    <#else>
        <#local testMaterialInformation="field not available"/>
    </#if>

    <#return testMaterialInformation/>
    
</#function>

<#-- no need to add species/strain from repeatable lists as only the species in materials/methods are taken, e.g.:
*ENDPOINT_STUDY_RECORD.GeneticToxicityVitro.ResultsAndDiscussion.TestRs.0.Organism
-->
<#function getSpecies doc>
    <#local species>
        <#compress>
        <#if doc.hasElement("MaterialsAndMethods.TestAnimals.Species")><#-- path for most Tox ESRs -->
            <@com.value doc.MaterialsAndMethods.TestAnimals.Species/>
        <#elseif doc.hasElement("MaterialsAndMethods.TestOrganisms.TestOrganismsSpecies")><#-- path for most Ecotox ESRs -->
            <@com.value doc.MaterialsAndMethods.TestOrganisms.TestOrganismsSpecies/>
        <#elseif doc.hasElement("MaterialsAndMethods.InVivoTestSystem.TestAnimals.Species")><#-- path for skin sensitisation -->
            <@com.value doc.MaterialsAndMethods.InVivoTestSystem.TestAnimals.Species/>
        <#elseif doc.hasElement("MaterialsAndMethods.Method.SpeciesStrain") && doc.MaterialsAndMethods.Method.SpeciesStrain?node_type=="repeatable">
            <#-- example to check the node type: node type: ${sectionDoc.MaterialsAndMethods.Method.SpeciesStrain?node_type} -->
            <#compress>
            <#list doc.MaterialsAndMethods.Method.SpeciesStrain as speciesRow>
                <@com.value valuePath=speciesRow.SpeciesStrain printDescription=false/><#if speciesRow?has_next> / </#if>
            </#list>
            </#compress>
        <#else>
            field not available
        </#if>
        </#compress>
    </#local>

    <#return species/>
</#function>

<#function getRouteApplicationDoseMethod doc>
    <#local routeApplicationDoseMethod><#compress>
        <#if doc.hasElement("MaterialsAndMethods.Studydesign.TestType")>
                <@com.value doc.MaterialsAndMethods.Studydesign.TestType/>
        <#elseif doc.hasElement("MaterialsAndMethods.TestMaterials.DoseMethod")>
                <@com.value doc.MaterialsAndMethods.TestMaterials.DoseMethod/>
        <#elseif doc.hasElement("MaterialsAndMethods.AdministrationExposure.RouteOfAdministration")>
                <@com.value doc.MaterialsAndMethods.AdministrationExposure.RouteOfAdministration/>
        <#elseif doc.hasElement("MaterialsAndMethods.TestSystem.TypeOfCoverage")>
                <@com.value doc.MaterialsAndMethods.TestSystem.TypeOfCoverage/>
        <#elseif doc.hasElement("MaterialsAndMethods.AdministrationExposure.TypeOfCoverage")>
                <@com.value doc.MaterialsAndMethods.AdministrationExposure.TypeOfCoverage/>
        <#else>
                field not available
        </#if>
    </#compress></#local>

    <#return routeApplicationDoseMethod/>
</#function>

<#function getExposureDuration doc>

    <#local exposureDuration><#compress>
        <#if doc.hasElement("MaterialsAndMethods.StudyDesign.TotalExposureDuration")>
            <@com.value doc.MaterialsAndMethods.StudyDesign.TotalExposureDuration/>
        
        <#elseif doc.hasElement("MaterialsAndMethods.AdministrationExposure.DurationAndFrequencyOfTreatmentExposure")>
            <@com.value doc.MaterialsAndMethods.AdministrationExposure.DurationAndFrequencyOfTreatmentExposure/>
        
        <#elseif doc.hasElement("MaterialsAndMethods.AdministrationExposure.DurationOfExposure")>
            <@com.value doc.MaterialsAndMethods.AdministrationExposure.DurationOfExposure/>
            
        <#elseif doc.hasElement("MaterialsAndMethods.TestSystem.DurationOfTreatmentExposure")>
            <@com.value doc.MaterialsAndMethods.TestSystem.DurationOfTreatmentExposure/>
            
        <#elseif doc.hasElement("MaterialsAndMethods.AdministrationExposure.DurationOfTreatmentExposure")>
            <@com.value doc.MaterialsAndMethods.AdministrationExposure.DurationOfTreatmentExposure/>

        <#elseif doc.hasElement("MaterialsAndMethods.StudyDesign.TotalExposureUptakeDuration")>
            <@com.value doc.MaterialsAndMethods.StudyDesign.TotalExposureUptakeDuration/>
                    
        </#if>
    </#compress></#local>

    <#return exposureDuration/>

</#function>

<#function getStudyResults doc resultsPathList>
    
    <#local endpointStudyRecordKeyResultsList = []/>
    <#local endpointStudyRecordAddResultsList = []/>
    
    <#local hasPath = false/>

    <#list resultsPathList as path>
        <#if doc.hasElement(path)>
            
            <#local hasPath = true/>

            <#local fullPath = 'doc.' + path/>
            <#local fullPath = fullPath?eval/>
            <#if fullPath?node_type == "repeatable">
                
                <#list fullPath as resultRow>                            
                
                    <#local resultsList = []/>
                    <#list resultRow?children as child>
                        <@iuclid.label for=child var="descript"/>

                        <#if child?has_content && !(descript?contains("Remark") || descript=="Key result")>
                            
                            <#local item>${descript}: <@compress single_line=true><@com.value child/></@compress></#local>
                            <#local resultsList = resultsList + [item]/>
                            </#if>
                    </#list>
                    <#local result = resultsList?join("; ")/>  
                        
                    <#if resultRow.hasElement("KeyResult") && resultRow.KeyResult>
                        <#local endpointStudyRecordKeyResultsList = endpointStudyRecordKeyResultsList + [result] />
                    <#else>
                        <#local endpointStudyRecordAddResultsList = endpointStudyRecordAddResultsList + [result] />
                    </#if>
                    
                </#list>
            </#if>
        </#if>
    </#list>

    <#local endpointStudyRecordKeyResults = escapeText(endpointStudyRecordKeyResultsList?join(" / "))/>
    <#local endpointStudyRecordAdditionalResults = escapeText(endpointStudyRecordAddResultsList?join(" / "))/>

    <#--  If there was no path found, then output "field not available"  -->
    <#if hasPath>
        <#return {'key results': endpointStudyRecordKeyResults, 'additional results': endpointStudyRecordAdditionalResults }/>
    <#else>
        <#return {'key results': "field not available", 'additional results': "field not available"}/>
    </#if>
</#function>

<#--  add: column with list of attachments as attachment name (brackets file type/format, size)  -->
<#function getLitRefFields ref>

    <#local litRefHash = {'LiteratureType':'', 'Name':'', 'Author':'', 'ReferenceYear':'', 'Source':'', 'TestLab':'', 'ReportDate':'', 'ReportNo':'', 'CompanyOwner':'', 'CompanyOwnerStudyNo':'', 'Remarks':'', 'StudyIdentifiers':'', "Attachments":""}/>
    <#if ref?has_content>
        <#local inf = ref.GeneralInfo/>

        <#list litRefHash?keys as key>
            <#if key == "StudyIdentifiers">
                <#local litRefHash = litRefHash + {'StudyIdentifiers' : getLitIds(ref)}/>
            
            <#elseif key == "Attachments">
                <#local litRefHash = litRefHash + {'Attachments' : getLitRefAttachments(ref)}/>

            <#else>
                <#local keyPath>inf.${key}</#local>
                <#local keyValue><@com.value keyPath?eval/></#local>
                <#local litRefHash = litRefHash + {key:keyValue}/>
            </#if>
        </#list>
        
    </#if>

    <#return litRefHash/>

</#function>

<#function getLitIds ref>
	<#local litIds>
		<@compress single_line=true>
			<#list ref.GeneralInfo.StudyIdentifiers as studyId>
				<#if studyId.StudyIDType?has_content><@com.value studyId.StudyIDType/>: </#if><@com.value studyId.StudyID/><#if studyId.Remarks?has_content> (<@com.value studyId.Remarks/>)</#if>
				<#if studyId?has_next> | </#if>
			</#list>
		</@compress>
	</#local>
	<#return litIds/>
</#function>

<#function getLitRefAttachments ref>

    <#local attachmentInfoList = []/>

    <#if ref.GeneralInfo.Attachments?has_content>
        <#list ref.GeneralInfo.Attachments as attachmentRow>
            
            <#local attachments = []/>
            <#if attachmentRow.AttachedDocuments?has_content>
                <#local attachments = attachments + [getAttachmentNameTypeSize(attachmentRow.AttachedDocuments, 'confidential')]/>
            </#if>
            <#if attachmentRow.AttachedSanitisedDocsForPublication?has_content>>
                <#local attachments = attachments + [getAttachmentNameTypeSize(attachmentRow.AttachedSanitisedDocsForPublication, 'sanitised')]/>
            </#if>

            <#local attachmentInfo>
                <#if attachmentRow.AttachmentType?has_content><@com.value attachmentRow.AttachmentType/>: </#if>${attachments?join("; ")}
            </#local>
            
            <#local attachmentInfoList = attachmentInfoList + [attachmentInfo]/>
        </#list>

    </#if>

    <#return attachmentInfoList?join(" | ")/>

</#function>

<#function getAttachmentNameTypeSize attachmentKey attachmentClass = 'confidential'>
  
  <#local attachmentMetaData = iuclid.getMetadataForAttachment(attachmentKey)>
    
  <#local attachmentNameTypeSize><@com.text attachmentMetaData.filename/> [${attachmentClass}] (type: ${attachmentMetaData.mediaType}, size:${attachmentMetaData.size?string["0.####"]})</#local>
  
  <#return attachmentNameTypeSize/>

</#function>

<#function escapeText text>
    <#if text?has_content>
        <#return text?replace("&gt;", ">")?replace("&lt;", "<")/>
    <#else>
        <#return text/>
    </#if>
</#function>

<#-- structure -->
    
<#--     ENTITY DOCUMENT -->
<#--        --- section node 1 -->
<#-- 		--- section node 1.1 -->
<#--             Section document 1 -->
<#--                 Lit ref 1 -->
<#--                 Lit ref 2 -->
<#--             Section document 2 -->
<#--                 Lit ref 3 -->
<#--         --- section node 2 -->
